##### Configuration

src_server ?= vegbiendev
src_user ?= $(USER)
test ?=

##### Vars/functions

# Make
null :=
space := $(null) $(null)
comma := ,
pathParts = $(shell path="$(1)"; echo "$${path%%/*}" "$${path\#*/}")
topDir = $(word 1,$(pathParts))
subPath = $(word 2,$(pathParts))
subMake = $(MAKE) $(call subPath,$@) --directory=$(call topDir,$@) \
--makefile=../input.Makefile
# input.Makefile path is relative to subdir

# Formatting
sed = sed -$(if $(filter Darwin,$(os)),E,r)
_2Space = $(subst _,$(space),$(1))
*2Space = $(call _2Space,$*)

# Filesystem
wildcard/ = $(shell echo $(1)|grep -vF '*')# emulate `shopt -s nullglob`
    # needed because builtin $(wildcard) doesn't do / suffix correctly
no/ = $(1:/=)# remove trailing /
+w = $(wildcard $+)# existing items in $+
CP := cp -p
cp = $(CP) $< $@

# System
os := $(shell uname)
forOs = $(patsubst %,%-$(filter Linux Darwin,$(os)),$(1))
date = $(shell date +"%Y-%m-%d-%H-%M-%S")

## SVN

addFile = $(if $(wildcard $(1)),,touch $(1) && )svn add $(1)
addDir = $(if $(wildcard $(1)/),svn add --depth=empty $(1),svn mkdir $(1))

# Revisions
revision = $(shell svn info $(1)|$(sed) -n 's/^Last Changed Rev: (.*)$$/\1/p')
rootRevision = $(call revision,$(root))
initRootRevision := $(if $(root),$(rootRevision))
rootRevisionsEqual := $(filter $(rootRevision),$(initRootRevision))
rootRevisions = $(foreach rootRevision,$(rootRevision),$(if\
$(rootRevisionsEqual),$(initRootRevision),$(initRootRevision)-$(rootRevision)))
    # only evaluate $(rootRevision) once
version = $(date).r$(rootRevisions)

# rsync
rsync* := "time" rsync$(if $(test), --dry-run) --archive --update --verbose\
--itemize-changes --progress
rsync := $(rsync*) --exclude=".*"
local := ./
localDirName := $(notdir $(realpath $(local)))
remote := $(src_user)@$(src_server):/home/bien/svn/$(localDirName)/

##### General targets

all: # must be first target in Makefile, so add an empty version here

.SUFFIXES: # turn off built-in suffix rules
.SECONDARY: # don't automatically delete intermediate files
.DELETE_ON_ERROR: # delete target if recipe fails

_always:
.PHONY: _always

clean: # make sure `make clean` always works
